/*

	mplayerplug-in
	
	This is a plugin for Netscape and Mozilla that uses mplayer to play supported videoes from websites
	It handles embedded and full page files equally.
	
	License: GPL
	
	Created 2003-01-09
	
	Version 0.10
	
	Author:			Changes:
	kdekorte		Initial Creation
	
	
	Contacts:
	kdekorte		Kevin DeKorte - kdekorte@yahoo.com

 *
 */

#include <stdio.h>
#include "npapi.h"


#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <signal.h>
#include <sys/wait.h>


#define MAX_BUF_LEN 255
#define STATE_NEW 1
#define STATE_HAVEURL 3
#define STATE_READY 5
#define DEBUG 1


typedef struct _PluginInstance
{
    int nothing;
    int state;
    char* url;
    char* fname;
    uint16 mode;
    Widget netscape_widget;
    Display* display;
    Window window;
    NPSetWindowCallbackStruct* ws;
    uint32 width;
    uint32 height;
    uint32 embed_width;
    uint32 embed_height;
    pid_t mplayer;
} PluginInstance;

void Redraw(Widget w, NPP instance, XEvent *event);


char*
NPP_GetMIMEDescription(void)
{
	/* type of files we handle */
	return("video/quicktime:mov:Quicktime;video/x-quicktime:mov:Quicktime;video/x-msvideo:avi:AVI;video/msvideo:avi:AVI;video/mpeg:mpg,mpeg:MPEG;video/x-mpeg:mpg,mpeg:MPEG;video/x-ms-asf-plugin:asf:Windows Media");
}

NPError
NPP_GetValue(void *future, NPPVariable variable, void *value)
{
	NPError err = NPERR_NO_ERROR;

	switch (variable) {
		case NPPVpluginNameString:
			*((char **)value) = "mplayerplug-in v0.10";
			break;
		case NPPVpluginDescriptionString:
			*((char **)value) =
				"Video Player Plug-in";
			break;
		default:
			err = NPERR_GENERIC_ERROR;
	}
	return err;
}

NPError
NPP_Initialize(void)
{
    return NPERR_NO_ERROR;
}


jref
NPP_GetJavaClass()
{
    return NULL;
}

void
NPP_Shutdown(void)
{
}


NPError
NPP_New(NPMIMEType pluginType,
	NPP instance,
	uint16 mode,
	int16 argc,
	char* argn[],
	char* argv[],
	NPSavedData* saved)
{
        PluginInstance* This;
	int i;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	instance->pdata = NPN_MemAlloc(sizeof(PluginInstance));

	This = (PluginInstance*) instance->pdata;
	//NPN_GetURL(instance, "javascript:document.location;","_self");

	This->url=NULL;
	This->fname=NULL;
	This->mode=mode;
	This->state=STATE_NEW;

	if (mode = NP_EMBED) {

		if (DEBUG) printf("Embedded mode\n");
		for(i=1;i<argc;i++){

			if(strstr(argn[i],"SRC") || strstr(argn[i],"src")){
				This->url=argv[i];
				This->state=STATE_HAVEURL;
				if (DEBUG) printf("\n==%s==%d\n",argv[i],i);
			}
		}
	}
	//printf("%i",NPN_GetURL(instance, "javascript:document.location;","_self"));

	if (This != NULL)
		return NPERR_NO_ERROR;
	else
		return NPERR_OUT_OF_MEMORY_ERROR;
}


NPError
NPP_Destroy(NPP instance, NPSavedData** save)
{

	PluginInstance* This;

	if (DEBUG) printf("In NPP_Destroy\n");

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	This = (PluginInstance*) instance->pdata;

	//kill(This->mplayer,SIGCHLD);

	if (This != NULL) {
		NPN_MemFree(instance->pdata);
		instance->pdata = NULL;
	}

	return NPERR_NO_ERROR;
}



NPError
NPP_SetWindow(NPP instance, NPWindow* window)
{
	PluginInstance* This;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	if (window == NULL)
		return NPERR_NO_ERROR;

	This = (PluginInstance*) instance->pdata;

	Widget netscape_widget;

	if (DEBUG) printf("Size: %d %d %x\n",window->x,window->y,window->window);
	if (DEBUG) printf("Size: %dx%d \n",window->width,window->height,window->window);

	NPSetWindowCallbackStruct* ws = (NPSetWindowCallbackStruct*) window->ws_info;

	netscape_widget = XtWindowToWidget((Display*) ws->display, (Window) window->window);

	This->display = ws->display;
	This->netscape_widget = netscape_widget;
	This->window = (Window)window->window;
	This->ws = ws;
	This->width = window->width;
	This->height = window->height;
	This->embed_width=window->x;
	This->embed_height=window->y;

	if (This->mode == NP_EMBED)
	{
		//XtAddEventHandler(netscape_widget, ExposureMask, FALSE, (XtEventHandler)Redraw, This);
		Redraw(This->netscape_widget, instance, NULL);
	}
	return NPERR_NO_ERROR;
}


NPError
NPP_NewStream(NPP instance,
			  NPMIMEType type,
			  NPStream *stream,
			  NPBool seekable,
			  uint16 *stype)
{
	NPByteRange range;
	PluginInstance* This;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	This = (PluginInstance*) instance->pdata;
	if (This->mode == NP_FULL) {
		*stype = NP_ASFILEONLY;
	}
	if (DEBUG) printf("NewStream Requested\n");
	if (DEBUG) printf("State %i \n",This->state);

	return NPERR_NO_ERROR;
}


/* PLUGIN DEVELOPERS:
 *	These next 2 functions are directly relevant in a plug-in which
 *	handles the data in a streaming manner. If you want zero bytes
 *	because no buffer space is YET available, return 0. As long as
 *	the stream has not been written to the plugin, Navigator will
 *	continue trying to send bytes.  If the plugin doesn't want them,
 *	just return some large number from NPP_WriteReady(), and
 *	ignore them in NPP_Write().  For a NP_ASFILE stream, they are
 *	still called but can safely be ignored using this strategy.
 */

int32 STREAMBUFSIZE = 0X0FFFFFFF; /* If we are reading from a file in NPAsFile
				   * mode so we can take any size stream in our
				   * write call (since we ignore it) */
int32
NPP_WriteReady(NPP instance, NPStream *stream)
{
	PluginInstance* This;
	if (instance != NULL)
		This = (PluginInstance*) instance->pdata;

	return STREAMBUFSIZE;
}


int32
NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *buffer)
{
	if (instance != NULL)
	{
		PluginInstance* This = (PluginInstance*) instance->pdata;

	}

	return len;		/* The number of bytes accepted */
}


NPError 
NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
{
	PluginInstance* This;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
	This = (PluginInstance*) instance->pdata;

	return NPERR_NO_ERROR;
}


void
NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
{
	PluginInstance* This;
	if (instance != NULL)
	{
		This = (PluginInstance*) instance->pdata;
	}
	FILE *infile;
	char c;
	char s[MAX_BUF_LEN];
	int i=0;

	if (DEBUG) printf("in stream as file. stream->url: %s\n",stream->url);
	if (DEBUG) printf("in stream as file. fname: %s\n",fname);

	if (This->url) free(This->url);
	This->url = strdup(stream->url);
	if (This->fname) free(This->fname);
	This->fname = strdup(fname);

	if (DEBUG) printf("set state\n");
	This->state=STATE_READY;

	Redraw(This->netscape_widget, instance, NULL);

}


void
NPP_Print(NPP instance, NPPrint* printInfo)
{
	return;
}

void
NPP_URLNotify(NPP instance, const char* url, NPReason reason, void* notifyData)
{
    PluginInstance* This;
    This = (PluginInstance*) instance->pdata;

    // welches GET-Request wurde abgearbeitet?
    if ((int) notifyData == 1)
        // falls ein Fehler aufgetreten ist
        if (reason != NPRES_DONE)
        {
        	if (DEBUG) printf("URL notify done: %s\n",url);
	}

}


void Redraw(Widget w, NPP instance, XEvent *event)
{

	if (DEBUG) printf("getting ready to fork mplayer\n");
        PluginInstance* This = (PluginInstance*)instance->pdata;

	if (DEBUG) printf("State: %i\n",This->state);

	if (This->mode == NP_EMBED && This->state == STATE_HAVEURL) {
		// we are ready
	} else {

		if (This->state != STATE_READY) {
			if (DEBUG) printf("Not ready to fork\n");
			return;
		}
	}

	GC gc;
        int pid;
        pid_t child, child2;
        char buffer[1000];
        XGCValues gcv;
        const char* text = "mplayer plug-in - Loading Movie..";
        if (DEBUG) printf("Window -- %x --\n",This->window);
	if (DEBUG) printf("URL -- %s\n",This->url);
	if (DEBUG) printf("File -- %s\n",This->fname);


	XtVaGetValues(w, XtNbackground, &gcv.background,XtNforeground, &gcv.foreground, 0);
	gc = XCreateGC((Display*)This->display, This->window, GCForeground|GCBackground, &gcv);
	XDrawRectangle((Display*)This->display, This->window, gc, 0, 0, This->width-1, This->height-1);
	XDrawString((Display*)This->display, This->window, gc, 10, 20,text, strlen(text));

	struct sigaction action;
	if (DEBUG) printf("\n\n ---------Forking------------ \n\n");
	child=fork();
	setsid();
	setpgid(child);
	if(child==0)
	{
		if (DEBUG) printf("\nsucessfully forked\n");
		if(strstr(This->url,"'")){
			if (DEBUG) printf("\n-------- ' is not allowed ------\n");
		} else {
			if (DEBUG) printf("\n-------- url seems to be ok ---\n");
			if (This->state == STATE_READY) {
				sprintf(buffer,"mplayer -vo xv,x11 -ao arts,oss -wid 0x%x -x %i -y %i %s",This->window, This->width, This->height, This->fname);
			} else {
				sprintf(buffer,"mplayer -vo xv,x11 -ao arts,oss -wid 0x%x %s",This->window,This->url);
			}
			child2=fork();
			if(child2==0){
				setsid();
				setpgid(child2);
				if (DEBUG) printf("%s",buffer);
				if (DEBUG) printf("\n\n---- forked PID %d ----",getppid());
				This->mplayer=child;
				system(buffer);
			} else {
				if(child2 < 0)
					abort();
			}
			_exit(0);
			//kill(child,SIGCHLD);
		}
		_exit(EXIT_FAILURE);
		//  wait();
	} else {
		if (child < 0)
			abort();
		else  {
			int status;
			wait(0);
		//             waitpid(-1,&status,WNOHANG);
			return;
		}
	}



}
